@@ -1,6 +1,6 @@
 ;; ede/source.el --- EDE source code object
 
-;; Copyright (C) 2000, 2008-2018 Free Software Foundation, Inc.
+;; Copyright (C) 2000, 2008-2021 Free Software Foundation, Inc.
 
 ;; Author: Eric M. Ludlam <zappo@gnu.org>
 ;; Keywords: project, make
@@ -156,14 +156,12 @@ Used to guess header files, but uses the auxsource regular expression."
 ;;
 ;; This must appear at the end so that the init method will work.
 (defvar ede-source-scheme
-  (ede-sourcecode "ede-source-scheme"
-		  :name "Scheme"
+  (ede-sourcecode :name "Scheme"
 		  :sourcepattern "\\.scm$")
   "Scheme source code definition.")
 
 ;;(defvar ede-source-
-;;  (ede-sourcecode "ede-source-"
-;;		    :name ""
+;;  (ede-sourcecode :name ""
 ;;		    :sourcepattern "\\.$"
 ;;		    :garbagepattern '("*."))
 ;;  " source code definition.")
