@@ -1,6 +1,6 @@
 ;;; semantic/fw.el --- Framework for Semantic
 
-;;; Copyright (C) 1999-2019 Free Software Foundation, Inc.
+;;; Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
 ;; Author: Eric M. Ludlam <zappo@gnu.org>
 
@@ -34,132 +34,56 @@
 
 ;;; Compatibility
 ;;
-(eval-and-compile
-  (if (featurep 'xemacs)
-      (progn
-	(defalias 'semantic-buffer-local-value 'symbol-value-in-buffer)
-        ;; FIXME: Why not just (require 'overlay)?
-	(defalias 'semantic-overlay-live-p
-	  (lambda (o)
-	    (and (extent-live-p o)
-		 (not (extent-detached-p o))
-		 (bufferp (extent-buffer o)))))
-	(defalias 'semantic-make-overlay
-	  (lambda (beg end &optional buffer &rest rest)
-	    "Xemacs `make-extent', supporting the front/rear advance options."
-	    (let ((ol (make-extent beg end buffer)))
-	      (when rest
-		(set-extent-property ol 'start-open (car rest))
-		(setq rest (cdr rest)))
-	      (when rest
-		(set-extent-property ol 'end-open (car rest)))
-	      ol)))
-	(defalias 'semantic-overlay-put             'set-extent-property)
-	(defalias 'semantic-overlay-get             'extent-property)
-	(defalias 'semantic-overlay-properties      'extent-properties)
-	(defalias 'semantic-overlay-move            'set-extent-endpoints)
-	(defalias 'semantic-overlay-delete          'delete-extent)
-	(defalias 'semantic-overlays-at
-	  (lambda (pos)
-	    (condition-case nil
-		(extent-list nil pos pos)
-	      (error nil))
-	    ))
-	(defalias 'semantic-overlays-in
-	  (lambda (beg end) (extent-list nil beg end)))
-	(defalias 'semantic-overlay-buffer          'extent-buffer)
-	(defalias 'semantic-overlay-start           'extent-start-position)
-	(defalias 'semantic-overlay-end             'extent-end-position)
-	(defalias 'semantic-overlay-size            'extent-length)
-	(defalias 'semantic-overlay-next-change     'next-extent-change)
-	(defalias 'semantic-overlay-previous-change 'previous-extent-change)
-	(defalias 'semantic-overlay-lists
-	  (lambda () (list (extent-list))))
-	(defalias 'semantic-overlay-p               'extentp)
-	(defalias 'semantic-event-window        'event-window)
-	(defun semantic-read-event ()
-	  (let ((event (next-command-event)))
-	    (if (key-press-event-p event)
-		(let ((c (event-to-character event)))
-		  (if (char-equal c (quit-char))
-		      (keyboard-quit)
-		    c)))
-	    event))
-	(defun semantic-popup-menu (menu)
-	  "Blocking version of `popup-menu'"
-	  (popup-menu menu)
-	  ;; Wait...
-	  (while (popup-up-p) (dispatch-event (next-event))))
-	)
-    ;; Emacs Bindings
-    (defalias 'semantic-overlay-live-p          'overlay-buffer)
-    (defalias 'semantic-make-overlay            'make-overlay)
-    (defalias 'semantic-overlay-put             'overlay-put)
-    (defalias 'semantic-overlay-get             'overlay-get)
-    (defalias 'semantic-overlay-properties      'overlay-properties)
-    (defalias 'semantic-overlay-move            'move-overlay)
-    (defalias 'semantic-overlay-delete          'delete-overlay)
-    (defalias 'semantic-overlays-at             'overlays-at)
-    (defalias 'semantic-overlays-in             'overlays-in)
-    (defalias 'semantic-overlay-buffer          'overlay-buffer)
-    (defalias 'semantic-overlay-start           'overlay-start)
-    (defalias 'semantic-overlay-end             'overlay-end)
-    (defalias 'semantic-overlay-next-change     'next-overlay-change)
-    (defalias 'semantic-overlay-previous-change 'previous-overlay-change)
-    (defalias 'semantic-overlay-lists           'overlay-lists)
-    (defalias 'semantic-overlay-p               'overlayp)
-    (defalias 'semantic-read-event              'read-event)
-    (defalias 'semantic-popup-menu              'popup-menu)
-    (defun semantic-event-window (event)
-      "Extract the window from EVENT."
-      (car (car (cdr event))))
+(define-obsolete-function-alias 'semantic-overlay-live-p 'overlay-buffer "27.1")
+(define-obsolete-function-alias 'semantic-make-overlay 'make-overlay "27.1")
+(define-obsolete-function-alias 'semantic-overlay-put 'overlay-put "27.1")
+(define-obsolete-function-alias 'semantic-overlay-get 'overlay-get "27.1")
+(define-obsolete-function-alias 'semantic-overlay-properties
+  'overlay-properties "27.1")
+(define-obsolete-function-alias 'semantic-overlay-move 'move-overlay "27.1")
+(define-obsolete-function-alias 'semantic-overlay-delete 'delete-overlay "27.1")
+(define-obsolete-function-alias 'semantic-overlays-at 'overlays-at "27.1")
+(define-obsolete-function-alias 'semantic-overlays-in 'overlays-in "27.1")
+(define-obsolete-function-alias 'semantic-overlay-buffer 'overlay-buffer "27.1")
+(define-obsolete-function-alias 'semantic-overlay-start 'overlay-start "27.1")
+(define-obsolete-function-alias 'semantic-overlay-end 'overlay-end "27.1")
+(define-obsolete-function-alias 'semantic-overlay-next-change
+  'next-overlay-change "27.1")
+(define-obsolete-function-alias 'semantic-overlay-previous-change
+  'previous-overlay-change "27.1")
+(define-obsolete-function-alias 'semantic-overlay-lists 'overlay-lists "27.1")
+(define-obsolete-function-alias 'semantic-overlay-p 'overlayp "27.1")
+(define-obsolete-function-alias 'semantic-read-event 'read-event "27.1")
+(define-obsolete-function-alias 'semantic-popup-menu 'popup-menu "27.1")
+(define-obsolete-function-alias 'semantic-buffer-local-value
+  'buffer-local-value "27.1")
 
-    (defalias 'semantic-buffer-local-value 'buffer-local-value)
+(defun semantic-event-window (event)
+  "Extract the window from EVENT."
+  (car (car (cdr event))))
 
-    )
+(define-obsolete-function-alias 'semantic-make-local-hook #'identity "27.1")
 
+(defalias 'semantic-mode-line-update #'force-mode-line-update)
 
-  (defalias 'semantic-make-local-hook
-    (if (featurep 'emacs)
-        #'identity  #'make-local-hook))
-
-  (defalias 'semantic-mode-line-update
-    (if (featurep 'xemacs) #'redraw-modeline #'force-mode-line-update))
-
-  ;; Since Emacs 22 major mode functions should use `run-mode-hooks' to
-  ;; run major mode hooks.
-  (defalias 'semantic-run-mode-hooks
-    (if (fboundp 'run-mode-hooks)
-	'run-mode-hooks
-      'run-hooks))
+;; Since Emacs 22 major mode functions should use `run-mode-hooks' to
+;; run major mode hooks.
+(defalias 'semantic-run-mode-hooks
+  (if (fboundp 'run-mode-hooks)
+      'run-mode-hooks
+    'run-hooks))
 
   ;; Fancy compat usage now handled in cedet-compat
-  (defalias 'semantic-subst-char-in-string 'subst-char-in-string)
-  )
+(defalias 'semantic-subst-char-in-string 'subst-char-in-string)
 
 (defun semantic-delete-overlay-maybe (overlay)
   "Delete OVERLAY if it is a semantic token overlay."
-  (if (semantic-overlay-get overlay 'semantic)
-      (semantic-overlay-delete overlay)))
+  (if (overlay-get overlay 'semantic)
+      (delete-overlay overlay)))
 
 ;;; Menu Item compatibility
 ;;
-(defun semantic-menu-item (item)
-  "Build an XEmacs compatible menu item from vector ITEM.
-That is remove the unsupported :help stuff."
-  (if (featurep 'xemacs)
-      (let ((n (length item))
-            (i 0)
-            slot l)
-        (while (< i n)
-          (setq slot (aref item i))
-          (if (and (keywordp slot)
-                   (eq slot :help))
-              (setq i (1+ i))
-            (setq l (cons slot l)))
-          (setq i (1+ i)))
-        (apply #'vector (nreverse l)))
-    item))
+(define-obsolete-function-alias 'semantic-menu-item #'identity "27.1")
 
 ;;; Positional Data Cache
 ;;
@@ -182,10 +106,10 @@ Possible Lifespans are:
   (or (memq lifespan '(end-of-command exit-cache-zone))
       (error "semantic-cache-data-to-buffer: Unknown LIFESPAN: %s"
              lifespan))
-  (let ((o (semantic-make-overlay start end buffer)))
-    (semantic-overlay-put o 'cache-name   name)
-    (semantic-overlay-put o 'cached-value value)
-    (semantic-overlay-put o 'lifespan     lifespan)
+  (let ((o (make-overlay start end buffer)))
+    (overlay-put o 'cache-name   name)
+    (overlay-put o 'cached-value value)
+    (overlay-put o 'lifespan     lifespan)
     (setq semantic-cache-data-overlays
           (cons o semantic-cache-data-overlays))
     ;;(message "Adding to cache: %s" o)
@@ -199,14 +123,14 @@ Remove self from `post-command-hook' if it is empty."
         (oldcache semantic-cache-data-overlays))
     (while oldcache
       (let* ((o    (car oldcache))
-             (life (semantic-overlay-get o 'lifespan))
+             (life (overlay-get o 'lifespan))
              )
         (if (or (eq life 'end-of-command)
                 (and (eq life 'exit-cache-zone)
-                     (not (member o (semantic-overlays-at (point))))))
+                     (not (member o (overlays-at (point))))))
             (progn
               ;;(message "Removing from cache: %s" o)
-              (semantic-overlay-delete o)
+              (delete-overlay o)
               )
           (setq newcache (cons o newcache))))
       (setq oldcache (cdr oldcache)))
@@ -221,14 +145,14 @@ Remove self from `post-command-hook' if it is empty."
   "Get cached data with NAME from optional POINT."
   (save-excursion
     (if point (goto-char point))
-    (let ((o (semantic-overlays-at (point)))
+    (let ((o (overlays-at (point)))
           (ans nil))
       (while (and (not ans) o)
-        (if (equal (semantic-overlay-get (car o) 'cache-name) name)
+        (if (equal (overlay-get (car o) 'cache-name) name)
             (setq ans (car o))
           (setq o (cdr o))))
       (when ans
-        (semantic-overlay-get ans 'cached-value)))))
+        (overlay-get ans 'cached-value)))))
 
 (defun semantic-test-data-cache ()
   "Test the data cache."
@@ -262,8 +186,8 @@ Mark OLDFNALIAS as obsolete, such that the byte compiler
 will throw a warning when it encounters this symbol."
   (defalias oldfnalias newfn)
   (make-obsolete oldfnalias newfn when)
-  (when (and (function-overload-p newfn)
-             (not (overload-obsoleted-by newfn))
+  (when (and (mode-local--function-overload-p newfn)
+             (not (mode-local--overload-obsoleted-by newfn))
              ;; Only throw this warning when byte compiling things.
              (boundp 'byte-compile-current-file)
              byte-compile-current-file
@@ -337,7 +261,7 @@ FUNCTION does not have arguments.  When FUNCTION is entered
 (semantic-alias-obsolete 'define-mode-overload-implementation
                          'define-mode-local-override "23.2")
 
-(defun semantic-install-function-overrides (overrides &optional transient mode)
+(defun semantic-install-function-overrides (overrides &optional transient)
   "Install the function OVERRIDES in the specified environment.
 OVERRIDES must be an alist ((OVERLOAD .  FUNCTION) ...) where OVERLOAD
 is a symbol identifying an overloadable entry, and FUNCTION is the
@@ -358,8 +282,7 @@ later installation should be done in MODE hook."
             (cons (intern (format "semantic-%s" name)) (cdr e)))))
     overrides)
    (list 'constant-flag (not transient)
-         'override-flag t)
-   mode))
+         'override-flag t)))
 
 ;;; User Interrupt handling
 ;;
@@ -403,11 +326,10 @@ calling this one."
 (defun semantic-find-file-noselect (file &optional nowarn rawfile wildcards)
   "Call `find-file-noselect' with various features turned off.
 Use this when referencing a file that will be soon deleted.
-FILE, NOWARN, RAWFILE, and WILDCARDS are passed into `find-file-noselect'"
+FILE, NOWARN, RAWFILE, and WILDCARDS are passed into `find-file-noselect'."
   ;; Hack -
   ;; Check if we are in set-auto-mode, and if so, warn about this.
-  (when (or  (and (featurep 'emacs) (boundp 'keep-mode-if-same))
-	     (and (featurep 'xemacs) (boundp 'just-from-file-name)))
+  (when (boundp 'keep-mode-if-same)
     (let ((filename (or (and (boundp 'filename) filename)
 			"(unknown)")))
       (message "WARNING: semantic-find-file-noselect called for \
@@ -439,10 +361,7 @@ into `mode-local-init-hook'." file filename)
 	 (enable-local-eval nil)
 	 )
     (save-match-data
-      (if (featurep 'xemacs)
-	  (find-file-noselect file nowarn rawfile)
-	(find-file-noselect file nowarn rawfile wildcards)))
-    ))
+      (find-file-noselect file nowarn rawfile wildcards))))
 
 ;;; Database restriction settings
 ;;
