@@ -1,9 +1,9 @@
-;;; gnus-gravatar.el --- Gnus Gravatar support
+;;; gnus-gravatar.el --- Gnus Gravatar support -*- lexical-binding: t -*-
 
-;; Copyright (C) 2010-2017 Free Software Foundation, Inc.
+;; Copyright (C) 2010-2021 Free Software Foundation, Inc.
 
 ;; Author: Julien Danjou <julien@danjou.info>
-;; Keywords: news
+;; Keywords: multimedia, news
 
 ;; This file is part of GNU Emacs.
 
@@ -29,13 +29,15 @@
 (require 'mail-extr) ;; Because of binding `mail-extr-disable-voodoo'.
 
 (defgroup gnus-gravatar nil
-  "Gnus Gravatar."
+  "Gravatars in Gnus."
+  :link '(custom-group-link gravatar)
   :group 'gnus-visual)
 
 (defcustom gnus-gravatar-size nil
-  "How big should gravatars be displayed.
+  "Size in pixels at which gravatars should be displayed.
 If nil, default to `gravatar-size'."
-  :type '(choice (const nil) integer)
+  :type '(choice (const :tag "Default" nil)
+                 (integer :tag "Pixels"))
   :version "24.1"
   :group 'gnus-gravatar)
 
@@ -46,8 +48,9 @@ If nil, default to `gravatar-size'."
   :group 'gnus-gravatar)
 
 (defcustom gnus-gravatar-too-ugly gnus-article-x-face-too-ugly
-  "Regexp matching posters whose avatar shouldn't be shown automatically."
-  :type '(choice regexp (const nil))
+  "Regexp matching posters whose avatar shouldn't be shown automatically.
+If nil, show all avatars."
+  :type '(choice regexp (const :tag "Allow all" nil))
   :version "24.1"
   :group 'gnus-gravatar)
 
@@ -73,56 +76,57 @@ If nil, default to `gravatar-size'."
 	  (ignore-errors
 	    (gravatar-retrieve
 	     (cadr address)
-	     'gnus-gravatar-insert
+             #'gnus-gravatar-insert
 	     (list header address category))))))))
 
 (defun gnus-gravatar-insert (gravatar header address category)
   "Insert GRAVATAR for ADDRESS in HEADER in current article buffer.
-Set image category to CATEGORY."
+Set image category to CATEGORY.  This function is intended as a
+callback for `gravatar-retrieve'."
   (unless (eq gravatar 'error)
     (gnus-with-article-buffer
-      (let ((mark (point-marker))
-	    (inhibit-point-motion-hooks t)
-	    (case-fold-search t))
-	(save-restriction
-	  (article-narrow-to-head)
-	  ;; The buffer can be gone at this time
-	  (when (buffer-live-p (current-buffer))
+      ;; The buffer can be gone at this time.
+      (when (buffer-live-p (current-buffer))
+        (let ((real-name (car address))
+              (mail-address (cadr address))
+              (mark (point-marker))
+              (inhibit-point-motion-hooks t)
+              (case-fold-search t))
+          (save-restriction
+            (article-narrow-to-head)
 	    (gnus-article-goto-header header)
 	    (mail-header-narrow-to-field)
-	    (let ((real-name (car address))
-		  (mail-address (cadr address)))
-	      (when (if real-name
-			(re-search-forward
-			 (concat (replace-regexp-in-string
-				  "[\t ]+" "[\t\n ]+"
-				  (regexp-quote real-name))
-				 "\\|"
-				 (regexp-quote mail-address))
-			 nil t)
-		      (search-forward mail-address nil t))
-		(goto-char (1- (match-beginning 0)))
-		;; If we're on the " quoting the name, go backward
-		(when (looking-at "[\"<]")
-		  (goto-char (1- (point))))
-		;; Do not do anything if there's already a gravatar. This can
-		;; happens if the buffer has been regenerated in the mean time, for
-		;; example we were fetching someaddress, and then we change to
-		;; another mail with the same someaddress.
-		(unless (memq 'gnus-gravatar (text-properties-at (point)))
-		  (let ((point (point)))
-		    (setq gravatar (append gravatar gnus-gravatar-properties))
-		    (gnus-put-image gravatar (buffer-substring (point) (1+ point)) category)
-		    (put-text-property point (point) 'gnus-gravatar address)
-		    (gnus-add-wash-type category)
-		    (gnus-add-image category gravatar)))))))
-	(goto-char (marker-position mark))))))
+            (when (if real-name
+                      (re-search-forward
+                       (concat (replace-regexp-in-string
+                                "[\t ]+" "[\t\n ]+"
+                                (regexp-quote real-name))
+                               "\\|"
+                               (regexp-quote mail-address))
+                       nil t)
+                    (search-forward mail-address nil t))
+              (goto-char (1- (match-beginning 0)))
+              ;; If we're on the " quoting the name, go backward.
+              (when (looking-at-p "[\"<]")
+                (goto-char (1- (point))))
+              ;; Do not do anything if there's already a gravatar.  This can
+              ;; happen if the buffer has been regenerated in the mean time, for
+              ;; example we were fetching someaddress, and then we change to
+              ;; another mail with the same someaddress.
+              (unless (get-text-property (point) 'gnus-gravatar)
+                (let ((pos (point)))
+                  (setq gravatar (append gravatar gnus-gravatar-properties))
+                  (gnus-put-image gravatar (buffer-substring pos (1+ pos)) category)
+                  (put-text-property pos (point) 'gnus-gravatar address)
+                  (gnus-add-wash-type category)
+                  (gnus-add-image category gravatar)))))
+          (goto-char mark))))))
 
 ;;;###autoload
 (defun gnus-treat-from-gravatar (&optional force)
   "Display gravatar in the From header.
 If gravatar is already displayed, remove it."
-  (interactive (list t)) ;; When type `W D g'
+  (interactive "p")
   (gnus-with-article-buffer
     (if (memq 'from-gravatar gnus-article-wash-types)
 	(gnus-delete-images 'from-gravatar)
@@ -132,12 +136,12 @@ If gravatar is already displayed, remove it."
 (defun gnus-treat-mail-gravatar (&optional force)
   "Display gravatars in the Cc and To headers.
 If gravatars are already displayed, remove them."
-  (interactive (list t)) ;; When type `W D h'
-    (gnus-with-article-buffer
-      (if (memq 'mail-gravatar gnus-article-wash-types)
-          (gnus-delete-images 'mail-gravatar)
-	(gnus-gravatar-transform-address "cc" 'mail-gravatar force)
-	(gnus-gravatar-transform-address "to" 'mail-gravatar force))))
+  (interactive "p")
+  (gnus-with-article-buffer
+    (if (memq 'mail-gravatar gnus-article-wash-types)
+        (gnus-delete-images 'mail-gravatar)
+      (gnus-gravatar-transform-address "cc" 'mail-gravatar force)
+      (gnus-gravatar-transform-address "to" 'mail-gravatar force))))
 
 (provide 'gnus-gravatar)
 
