@@ -1,6 +1,6 @@
 ;;; nnrss.el --- interfacing with RSS
 
-;; Copyright (C) 2001-2019 Free Software Foundation, Inc.
+;; Copyright (C) 2001-2021 Free Software Foundation, Inc.
 
 ;; Author: Shenghuo Zhu <zsh@cs.rochester.edu>
 ;; Keywords: RSS
@@ -24,7 +24,7 @@
 
 ;;; Code:
 
-(eval-when-compile (require 'cl))
+(eval-when-compile (require 'cl-lib))
 
 (require 'gnus)
 (require 'nnoo)
@@ -36,8 +36,10 @@
 (require 'rfc2231)
 (require 'mm-url)
 (require 'rfc2047)
+(require 'iso8601)
 (require 'mml)
 (require 'xml)
+(require 'dom)
 
 (defgroup nnrss nil
   "RSS access for Gnus."
@@ -49,7 +51,7 @@
   "Where nnrss will save its files.")
 
 (defvoo nnrss-ignore-article-fields '(slash:comments)
-  "*List of fields that should be ignored when comparing RSS articles.
+  "List of fields that should be ignored when comparing RSS articles.
 Some RSS feeds update article fields during their lives, e.g. to
 indicate the number of comments or the number of times the
 articles have been seen.  However, if there is a difference
@@ -340,10 +342,10 @@ for decoding when the cdr that the data specify is not available.")
   (let (elem)
     ;; There may be two or more entries in `nnrss-group-alist' since
     ;; this function didn't delete them formerly.
-    (while (setq elem (assoc group nnrss-group-alist))
+    (while (setq elem (assoc-string group nnrss-group-alist))
       (setq nnrss-group-alist (delq elem nnrss-group-alist))))
   (setq nnrss-server-data
-	(delq (assoc group nnrss-server-data) nnrss-server-data))
+	(delq (assoc-string group nnrss-server-data) nnrss-server-data))
   (nnrss-save-server-data server)
   (ignore-errors
     (let ((file-name-coding-system nnmail-pathname-coding-system))
@@ -355,8 +357,8 @@ for decoding when the cdr that the data specify is not available.")
   (with-current-buffer nntp-server-buffer
     (erase-buffer)
     (dolist (elem nnrss-group-alist)
-      (if (third elem)
-	  (insert (car elem) "\t" (third elem) "\n"))))
+      (if (nth 2 elem)
+	  (insert (car elem) "\t" (nth 2 elem) "\n"))))
   t)
 
 (deffoo nnrss-retrieve-groups (groups &optional server)
@@ -367,7 +369,7 @@ for decoding when the cdr that the data specify is not available.")
   (with-current-buffer nntp-server-buffer
     (erase-buffer)
     (dolist (group groups)
-      (let ((elem (assoc (gnus-group-decoded-name group) nnrss-server-data)))
+      (let ((elem (assoc-string group nnrss-server-data)))
 	(insert (format "%S %s 1 y\n" group (or (cadr elem) 0)))))
     'active))
 
@@ -446,16 +448,16 @@ nnrss: %s: Not valid XML %s and libxml-parse-html-region doesn't work %s"
 (autoload 'timezone-parse-date "timezone")
 
 (defun nnrss-normalize-date (date)
-  "Return a date string of DATE in the RFC822 style.
+  "Return a date string of DATE in the style of RFC 822 and its successors.
 This function handles the ISO 8601 date format described in
-URL `http://www.w3.org/TR/NOTE-datetime', and also the RFC822 style
+URL `http://www.w3.org/TR/NOTE-datetime', and also the RFC 822 style
 which RSS 2.0 allows."
   (let (case-fold-search vector year month day time zone cts given)
     (cond ((null date))			; do nothing for this case
 	  ;; if the date is just digits (unix time stamp):
 	  ((string-match "^[0-9]+$" date)
-	   (setq given (seconds-to-time (string-to-number date))))
-	  ;; RFC822
+	   (setq given (time-convert (string-to-number date))))
+	  ;; RFC 822
 	  ((string-match " [0-9]+ " date)
 	   (setq vector (timezone-parse-date date)
 		 year (string-to-number (aref vector 0)))
@@ -468,49 +470,25 @@ which RSS 2.0 allows."
 			(not (string-match "\\`[A-Z+-]" zone)))
 	       (setq zone nil))))
 	  ;; ISO 8601
-	  ((string-match
-	    (eval-when-compile
-	      (concat
-	       ;; 1. year
-	       "\\(199[0-9]\\|20[0-9][0-9]\\)"
-	       "\\(?:-"
-	       ;; 2. month
-	       "\\([01][0-9]\\)"
-	       "\\(?:-"
-	       ;; 3. day
-	       "\\([0-3][0-9]\\)"
-	       "\\)?\\)?\\(?:T"
-	       ;; 4. hh:mm
-	       "\\([012][0-9]:[0-5][0-9]\\)"
-	       "\\(?:"
-	       ;; 5. :ss
-	       "\\(:[0-5][0-9]\\)"
-	       "\\(?:\\.[0-9]+\\)?\\)?\\)?"
-	       ;; 6+7,8,9. zone
-	       "\\(?:\\(?:\\([+-][012][0-9]\\):\\([0-5][0-9]\\)\\)"
-	       "\\|\\([+-][012][0-9][0-5][0-9]\\)"
-	       "\\|\\(Z\\)\\)?"))
-	    date)
-	   (setq year (string-to-number (match-string 1 date))
-		 month (string-to-number (or (match-string 2 date) "1"))
-		 day (string-to-number (or (match-string 3 date) "1"))
-		 time (if (match-beginning 5)
-			  (substring date (match-beginning 4) (match-end 5))
-			(concat (or (match-string 4 date) "00:00") ":00"))
-		 zone (cond ((match-beginning 6)
-			     (concat (match-string 6 date)
-				     (match-string 7 date)))
-			    ((match-beginning 9) ;; Z
-			     "+0000")
-			    (t ;; nil if zone is not provided.
-			     (match-string 8 date))))))
+	  ((iso8601-valid-p date)
+	   (let ((decoded (decoded-time-set-defaults (iso8601-parse date))))
+	     (setq year (decoded-time-year decoded)
+		   month (decoded-time-month decoded)
+		   day (decoded-time-day decoded)
+		   time (format "%02d:%02d:%02d"
+				(decoded-time-hour decoded)
+				(decoded-time-minute decoded)
+				(decoded-time-second decoded))
+		   zone (if (equal (decoded-time-zone decoded) "Z")
+			    0
+			  (decoded-time-zone decoded))))))
     (if month
 	(progn
 	  (setq cts (current-time-string (encode-time 0 0 0 day month year)))
 	  (format "%s, %02d %s %04d %s%s"
 		  (substring cts 0 3) day (substring cts 4 7) year time
 		  (if zone
-		      (concat " " zone)
+		      (concat " " (format-time-string "%z" nil zone))
 		    "")))
       (message-make-date given))))
 
@@ -539,7 +517,7 @@ which RSS 2.0 allows."
   (if (hash-table-p nnrss-group-hashtb)
       (clrhash nnrss-group-hashtb)
     (setq nnrss-group-hashtb (make-hash-table :test 'equal)))
-  (let ((pair (assoc group nnrss-server-data)))
+  (let ((pair (assoc-string group nnrss-server-data)))
     (setq nnrss-group-max (or (cadr pair) 0))
     (setq nnrss-group-min (+ nnrss-group-max 1)))
   (let ((file (nnrss-make-filename group server))
@@ -625,7 +603,7 @@ which RSS 2.0 allows."
 ;;; Snarf functions
 (defun nnrss-make-hash-index (item)
   (gnus-message 9 "nnrss: Making hash index of %s" (gnus-prin1-to-string item))
-  (setq item (gnus-remove-if
+  (setq item (seq-remove
 	      (lambda (field)
 		(when (listp field)
 		  (memq (car field) nnrss-ignore-article-fields)))
@@ -644,8 +622,8 @@ which RSS 2.0 allows."
 					 (concat group ".xml"))
 					nnrss-directory))))
 	(setq xml (nnrss-fetch file t))
-      (setq url (or (nth 2 (assoc group nnrss-server-data))
-		    (second (assoc group nnrss-group-alist))))
+      (setq url (or (nth 2 (assoc-string group nnrss-server-data))
+		    (cadr (assoc-string group nnrss-group-alist))))
       (unless url
 	(setq url
 	      (cdr
@@ -653,7 +631,7 @@ which RSS 2.0 allows."
 		      (nnrss-discover-feed
 		       (read-string
 			(format "URL to search for %s: " group) "http://")))))
-	(let ((pair (assoc group nnrss-server-data)))
+	(let ((pair (assoc-string group nnrss-server-data)))
 	  (if pair
 	      (setcdr (cdr pair) (list url))
 	    (push (list group nnrss-group-max url) nnrss-server-data)))
@@ -691,7 +669,7 @@ which RSS 2.0 allows."
 		  (if (and len (integerp (setq len (string-to-number len))))
 		      ;; actually already in `ls-lisp-format-file-size' but
 		      ;; probably not worth to require it for one function
-		      (do ((size (/ len 1.0) (/ size 1024.0))
+		      (cl-do ((size (/ len 1.0) (/ size 1024.0))
 			   (post-fixes (list "" "k" "M" "G" "T" "P" "E")
 				       (cdr post-fixes)))
 			  ((< size 1024)
@@ -705,7 +683,7 @@ which RSS 2.0 allows."
 	    (setq enclosure (list url name len type))))
 	(push
 	 (list
-	  (incf nnrss-group-max)
+	  (cl-incf nnrss-group-max)
 	  (current-time)
 	  url
 	  (and subject (nnrss-mime-encode-string subject))
@@ -721,7 +699,7 @@ which RSS 2.0 allows."
       (setq extra nil))
     (when changed
       (nnrss-save-group-data group server)
-      (let ((pair (assoc group nnrss-server-data)))
+      (let ((pair (assoc-string group nnrss-server-data)))
 	(if pair
 	    (setcar (cdr pair) nnrss-group-max)
 	  (push (list group nnrss-group-max) nnrss-server-data)))
@@ -792,7 +770,7 @@ It is useful when `(setq nnrss-use-local t)'."
   (insert "RSSDIR='" (expand-file-name nnrss-directory) "'\n")
   (dolist (elem nnrss-server-data)
     (let ((url (or (nth 2 elem)
-		   (second (assoc (car elem) nnrss-group-alist)))))
+		   (cadr (assoc-string (car elem) nnrss-group-alist)))))
       (insert "$WGET -q -O \"$RSSDIR\"/'"
 	      (nnrss-translate-file-chars (concat (car elem) ".xml"))
 	      "' '" url "'\n"))))
@@ -1031,7 +1009,11 @@ Simply ensures that the first element is rss or rdf."
   "Given EL (containing a parsed element) and URI (containing a string
 that gives the URI for which you want to retrieve the namespace
 prefix), return the prefix."
-  (let* ((prefix (car (rassoc uri (cadar el))))
+  (let* ((prefix (car (rassoc uri (dom-attributes
+				   (dom-search
+				    el
+				    (lambda (node)
+				      (rassoc uri (dom-attributes node))))))))
 	 (nslist (if prefix
 		     (split-string (symbol-name prefix) ":")))
 	 (ns (cond ((eq (length nslist) 1) ; no prefix given
