@@ -1,6 +1,6 @@
 ;;; ibuf-ext.el --- extensions for ibuffer  -*- lexical-binding:t -*-
 
-;; Copyright (C) 2000-2019 Free Software Foundation, Inc.
+;; Copyright (C) 2000-2021 Free Software Foundation, Inc.
 
 ;; Author: Colin Walters <walters@verbum.org>
 ;; Maintainer: John Paul Wallington <jpw@gnu.org>
@@ -127,12 +127,12 @@ Buffers whose major mode is in this list, are not searched."
 (defvar ibuffer-auto-buffers-changed nil)
 
 (defun ibuffer-update-saved-filters-format (filters)
-  "Transforms alist from old to new `ibuffer-saved-filters' format.
+  "Transform alist from old to new `ibuffer-saved-filters' format.
 
 Specifically, converts old-format alist with values of the
 form (STRING (FILTER-SPECS...)) to alist with values of the
 form (STRING FILTER-SPECS...), where each filter spec should be a
-cons cell with a symbol in the car. Any elements in the latter
+cons cell with a symbol in the car.  Any elements in the latter
 form are kept as is.
 
 Returns (OLD-FORMAT-DETECTED . UPDATED-SAVED-FILTERS-LIST)."
@@ -178,14 +178,14 @@ Returns (OLD-FORMAT-DETECTED . UPDATED-SAVED-FILTERS-LIST)."
 
 Each element should look like (\"NAME\" . FILTER-LIST), where
 FILTER-LIST has the same structure as the variable
-`ibuffer-filtering-qualifiers', which see. The filters defined
+`ibuffer-filtering-qualifiers', which see.  The filters defined
 here are joined with an implicit logical `and' and associated
-with NAME. The combined specification can be used by name in
+with NAME.  The combined specification can be used by name in
 other filter specifications via the `saved' qualifier (again, see
 `ibuffer-filtering-qualifiers'). They can also be switched to by
 name (see the functions `ibuffer-switch-to-saved-filters' and
-`ibuffer-save-filters'). The variable `ibuffer-save-with-custom'
-affects how this information is saved for future sessions. This
+`ibuffer-save-filters').  The variable `ibuffer-save-with-custom'
+affects how this information is saved for future sessions.  This
 variable can be set directly from lisp code."
   :version "26.1"
   :type '(alist :key-type (string :tag "Filter name")
@@ -213,14 +213,14 @@ either clicking or hitting return "
                        (customize-save-variable 'ibuffer-saved-filters
                                                 ibuffer-saved-filters)
                        (message "Saved updated ibuffer-saved-filters."))))
-          ". See below for
+          ".  See below for
 an explanation and alternative ways to save the repaired value.
 
 Explanation: For the list variable `ibuffer-saved-filters',
 elements of the form (STRING (FILTER-SPECS...)) are deprecated
 and should instead have the form (STRING FILTER-SPECS...), where
-each filter spec is a cons cell with a symbol in the car. See
-`ibuffer-saved-filters' for details. The repaired value fixes
+each filter spec is a cons cell with a symbol in the car.  See
+`ibuffer-saved-filters' for details.  The repaired value fixes
 this format without changing the meaning of the saved filters.
 
 Alternative ways to save the repaired value:
@@ -238,7 +238,7 @@ Alternative ways to save the repaired value:
   "A list specifying the filters currently acting on the buffer list.
 
 If this list is nil, then no filters are currently in
-effect. Otherwise, each element of this list specifies a single
+effect.  Otherwise, each element of this list specifies a single
 filter, and all of the specified filters in the list are applied
 successively to the buffer list.
 
@@ -273,7 +273,7 @@ A compound filter specification can have one of four forms:
 -- (saved . \"NAME\")
 
    Represents the filter saved under the string NAME
-   in the alist `ibuffer-saved-filters'. It is an
+   in the alist `ibuffer-saved-filters'.  It is an
    error to name a filter that has not been saved.
 
 This variable is local to each ibuffer buffer.")
@@ -363,12 +363,12 @@ Currently, this only applies to `ibuffer-saved-filters' and
   :group 'ibuffer)
 
 (defun ibuffer-repair-saved-filters ()
-  "Updates `ibuffer-saved-filters' to its new-style format, if needed.
+  "Update `ibuffer-saved-filters' to its new-style format, if needed.
 
 If this list has any elements of the old-style format, a
 deprecation warning is raised, with a button allowing persistent
-update. Any updated filters retain their meaning in the new
-format. See `ibuffer-update-saved-filters-format' and
+update.  Any updated filters retain their meaning in the new
+format.  See `ibuffer-update-saved-filters-format' and
 `ibuffer-saved-filters' for details of the old and new formats."
   (interactive)
   (when (and (boundp 'ibuffer-saved-filters) ibuffer-saved-filters)
@@ -403,10 +403,7 @@ format. See `ibuffer-update-saved-filters-format' and
 
 ;;;###autoload
 (define-minor-mode ibuffer-auto-mode
-  "Toggle use of Ibuffer's auto-update facility (Ibuffer Auto mode).
-With a prefix argument ARG, enable Ibuffer Auto mode if ARG is
-positive, and disable it otherwise.  If called from Lisp, enable
-the mode if ARG is omitted or nil."
+  "Toggle use of Ibuffer's auto-update facility (Ibuffer Auto mode)."
   nil nil nil
   (unless (derived-mode-p 'ibuffer-mode)
     (error "This buffer is not in Ibuffer mode"))
@@ -563,13 +560,10 @@ the mode if ARG is omitted or nil."
 ;;;###autoload (autoload 'ibuffer-do-eval "ibuf-ext")
 (define-ibuffer-op eval (form)
   "Evaluate FORM in each of the buffers.
-Does not display the buffer during evaluation. See
-`ibuffer-do-view-and-eval' for that."
+Does not display the buffer during evaluation.
+See `ibuffer-do-view-and-eval' for that."
   (:interactive
-   (list
-    (read-from-minibuffer
-     "Eval in buffers (form): "
-     nil read-expression-map t 'read-expression-history))
+   (list (read--expression "Eval in buffers (form): "))
    :opstring "evaluated in"
    :modifier-p :maybe)
   (eval form))
@@ -700,10 +694,10 @@ specifications with the same structure as
                  filters))))
 
 (defun ibuffer-unary-operand (filter)
-  "Extracts operand from a unary compound FILTER specification.
+  "Extract operand from a unary compound FILTER specification.
 
 FILTER should be a cons cell of either form (f . d) or (f d),
-where operand d is itself a cons cell, or nil. Returns d."
+where operand d is itself a cons cell, or nil.  Returns d."
   (let* ((tail (cdr filter))
          (maybe-q (car-safe tail)))
     (if (consp maybe-q) maybe-q tail)))
@@ -726,7 +720,7 @@ specification, with the same structure as an element of the list
   (not
    (not
     (pcase (car filter)
-      (`or
+      ('or
        ;;; ATTN: Short-circuiting alternative with parallel structure w/`and
        ;;(catch 'has-match
        ;;  (dolist (filter-spec (cdr filter) nil)
@@ -735,12 +729,12 @@ specification, with the same structure as an element of the list
        (memq t (mapcar #'(lambda (x)
                            (ibuffer-included-in-filter-p buf x))
                        (cdr filter))))
-      (`and
+      ('and
        (catch 'no-match
          (dolist (filter-spec (cdr filter) t)
            (unless (ibuffer-included-in-filter-p buf filter-spec)
              (throw 'no-match nil)))))
-      (`saved
+      ('saved
        (let ((data (assoc (cdr filter) ibuffer-saved-filters)))
 	 (unless data
 	   (ibuffer-filter-disable t)
@@ -766,12 +760,11 @@ specification, with the same structure as an element of the list
 	  (i 0))
       (dolist (filtergroup filter-group-alist)
 	(let ((filterset (cdr filtergroup)))
-	  (cl-multiple-value-bind (hip-crowd lamers)
-	      (cl-values-list
-	       (ibuffer-split-list (lambda (bufmark)
-				     (ibuffer-included-in-filters-p (car bufmark)
-								    filterset))
-				   bmarklist))
+	  (cl-destructuring-bind (hip-crowd lamers)
+	      (ibuffer-split-list (lambda (bufmark)
+				    (ibuffer-included-in-filters-p (car bufmark)
+								   filterset))
+				  bmarklist)
 	    (aset vec i hip-crowd)
 	    (cl-incf i)
 	    (setq bmarklist lamers))))
@@ -1033,8 +1026,11 @@ group definitions by setting `ibuffer-filter-groups' to nil."
       (ibuffer-jump-to-buffer (buffer-name buf)))))
 
 (defun ibuffer-push-filter (filter-specification)
-  "Add FILTER-SPECIFICATION to `ibuffer-filtering-qualifiers'."
-  (push filter-specification ibuffer-filtering-qualifiers))
+  "Add FILTER-SPECIFICATION to `ibuffer-filtering-qualifiers'.
+If FILTER-SPECIFICATION is already in the list then return nil.  Otherwise,
+return the updated list."
+  (unless (member filter-specification ibuffer-filtering-qualifiers)
+    (push filter-specification ibuffer-filtering-qualifiers)))
 
 ;;;###autoload
 (defun ibuffer-decompose-filter ()
@@ -1051,14 +1047,14 @@ turned into separate filters, like [name: foo] and [mode: bar-mode]."
          (tail (cdr filters))
          (value
           (pcase (caar filters)
-            ((or `or 'and) (nconc head tail))
-            (`saved
+            ((or 'or 'and) (nconc head tail))
+            ('saved
              (let ((data (assoc head ibuffer-saved-filters)))
                (unless data
                  (ibuffer-filter-disable)
                  (error "Unknown saved filter %s" head))
                (append (cdr data) tail)))
-            (`not (cons (ibuffer-unary-operand (car filters)) tail))
+            ('not (cons (ibuffer-unary-operand (car filters)) tail))
             (_
              (error "Filter type %s is not compound" (caar filters))))))
     (setq ibuffer-filtering-qualifiers value))
@@ -1197,12 +1193,12 @@ Interactively, prompt for NAME, and use the current filters."
 
 (defun ibuffer-format-qualifier-1 (qualifier)
   (pcase (car qualifier)
-    (`saved
+    ('saved
      (concat " [filter: " (cdr qualifier) "]"))
-    (`or
+    ('or
      (concat " [OR" (mapconcat #'ibuffer-format-qualifier
                                (cdr qualifier) "") "]"))
-    (`and
+    ('and
      (concat " [AND" (mapconcat #'ibuffer-format-qualifier
                                 (cdr qualifier) "") "]"))
     (_
@@ -1228,28 +1224,33 @@ If INCLUDE-PARENTS is non-nil then include parent modes."
 
 ;;;###autoload (autoload 'ibuffer-filter-by-mode "ibuf-ext")
 (define-ibuffer-filter mode
-  "Limit current view to buffers with major mode QUALIFIER."
+    "Limit current view to buffers with major mode(s) specified by QUALIFIER.
+QUALIFIER is the mode name as a symbol or a list of symbols.
+Called interactively, accept a comma separated list of mode names."
   (:description "major mode"
    :reader
    (let* ((buf (ibuffer-current-buffer))
           (default (if (and buf (buffer-live-p buf))
                        (symbol-name (buffer-local-value
                                      'major-mode buf)))))
-     (intern
-      (completing-read
+     (mapcar #'intern
+      (completing-read-multiple
        (if default
            (format "Filter by major mode (default %s): " default)
          "Filter by major mode: ")
        obarray
-       #'(lambda (e)
-           (string-match "-mode\\'" (symbol-name e)))
-       t nil nil default))))
+       (lambda (e)
+           (string-match "-mode\\'" (if (symbolp e) (symbol-name e) e)))
+       t nil nil default)))
+   :accept-list t)
   (eq qualifier (buffer-local-value 'major-mode buf)))
 
 ;;;###autoload (autoload 'ibuffer-filter-by-used-mode "ibuf-ext")
 (define-ibuffer-filter used-mode
-  "Limit current view to buffers with major mode QUALIFIER.
-Called interactively, this function allows selection of modes
+    "Limit current view to buffers with major mode(s) specified by QUALIFIER.
+QUALIFIER is the mode name as a symbol or a list of symbols.
+
+Called interactively, accept a comma separated list of mode names
 currently used by buffers."
   (:description "major mode in use"
    :reader
@@ -1257,23 +1258,29 @@ currently used by buffers."
           (default (if (and buf (buffer-live-p buf))
                        (symbol-name (buffer-local-value
                                      'major-mode buf)))))
-     (intern
-      (completing-read
+     (mapcar #'intern
+      (completing-read-multiple
        (if default
            (format "Filter by major mode (default %s): " default)
          "Filter by major mode: ")
-       (ibuffer-list-buffer-modes) nil t nil nil default))))
+       (ibuffer-list-buffer-modes) nil t nil nil default)))
+   :accept-list t)
   (eq qualifier (buffer-local-value 'major-mode buf)))
 
 ;;;###autoload (autoload 'ibuffer-filter-by-derived-mode "ibuf-ext")
 (define-ibuffer-filter derived-mode
-    "Limit current view to buffers whose major mode inherits from QUALIFIER."
+    "Limit current view to buffers with major mode(s) specified by QUALIFIER.
+QUALIFIER is the mode name as a symbol or a list of symbols.
+ Restrict the view to buffers whose major mode derivates
+ from modes specified by QUALIFIER.
+Called interactively, accept a comma separated list of mode names."
   (:description "derived mode"
-		:reader
-		(intern
-		 (completing-read "Filter by derived mode: "
-				  (ibuffer-list-buffer-modes t)
-                                  nil t)))
+        :reader
+        (mapcar #'intern
+         (completing-read-multiple "Filter by derived mode: "
+                       (ibuffer-list-buffer-modes t)
+                       nil t))
+        :accept-list t)
   (with-current-buffer buf (derived-mode-p qualifier)))
 
 ;;;###autoload (autoload 'ibuffer-filter-by-name "ibuf-ext")
@@ -1283,6 +1290,12 @@ currently used by buffers."
    :reader (read-from-minibuffer "Filter by name (regexp): "))
   (string-match qualifier (buffer-name buf)))
 
+;;;###autoload (autoload 'ibuffer-filter-by-process "ibuf-ext")
+(define-ibuffer-filter process
+    "Limit current view to buffers running a process."
+  (:description "process")
+  (get-buffer-process buf))
+
 ;;;###autoload (autoload 'ibuffer-filter-by-starred-name "ibuf-ext")
 (define-ibuffer-filter starred-name
     "Limit current view to buffers with name beginning and ending
@@ -1307,8 +1320,8 @@ matches against '/a/b/c.d'."
 (define-ibuffer-filter basename
     "Limit current view to buffers with file basename matching QUALIFIER.
 
-For example, for a buffer associated with file '/a/b/c.d', this
-matches against 'c.d'."
+For example, for a buffer associated with file `/a/b/c.d', this
+matches against `c.d'."
   (:description "file basename"
    :reader (read-from-minibuffer
             "Filter by file name, without directory part (regex): "))
@@ -1320,8 +1333,8 @@ matches against 'c.d'."
     "Limit current view to buffers with filename extension matching QUALIFIER.
 
 The separator character (typically `.') is not part of the
-pattern. For example, for a buffer associated with file
-'/a/b/c.d', this matches against 'd'."
+pattern.  For example, for a buffer associated with file
+`/a/b/c.d', this matches against `d'."
   (:description "filename extension"
    :reader (read-from-minibuffer
             "Filter by filename extension without separator (regex): "))
@@ -1333,7 +1346,7 @@ pattern. For example, for a buffer associated with file
     "Limit current view to buffers with directory matching QUALIFIER.
 
 For a buffer associated with file '/a/b/c.d', this matches
-against '/a/b'. For a buffer not associated with a file, this
+against '/a/b'.  For a buffer not associated with a file, this
 matches against the value of `default-directory' in that buffer."
   (:description "directory name"
    :reader (read-from-minibuffer "Filter by directory name (regex): "))
@@ -1582,7 +1595,7 @@ to move by.  The default is `ibuffer-marked-char'."
   "Hide all of the currently marked lines."
   (interactive)
   (if (= (ibuffer-count-marked-lines) 0)
-      (message "No buffers marked; use 'm' to mark a buffer")
+      (message "No buffers marked; use `m' to mark a buffer")
     (let ((count
 	   (ibuffer-map-marked-lines
 	    #'(lambda (_buf _mark)
@@ -1829,7 +1842,8 @@ When BUF nil, default to the buffer at current line."
 			  (stringp dired-directory)
 			  dired-directory)))))
 	 (when name
-	   (string-match regexp name))))))
+           ;; Match on the displayed file name (which is abbreviated).
+	   (string-match regexp (abbreviate-file-name name)))))))
 
 ;;;###autoload
 (defun ibuffer-mark-by-content-regexp (regexp &optional all-buffers)
@@ -1847,8 +1861,9 @@ Otherwise buffers whose name matches an element of
          (cond ((and (not all-buffers)
                      (or
                       (memq mode ibuffer-never-search-content-mode)
-                      (cl-some (lambda (x) (string-match x (buffer-name buf)))
-                               ibuffer-never-search-content-name)))
+                      (cl-dolist (x ibuffer-never-search-content-name nil)
+                        (when-let ((found (string-match x (buffer-name buf))))
+                          (cl-return found)))))
                 (setq res nil))
                (t
                 (with-current-buffer buf
@@ -1931,11 +1946,10 @@ Otherwise buffers whose name matches an element of
   (ibuffer-mark-on-buffer
    #'(lambda (buf)
        (with-current-buffer buf
-	 ;; hacked from midnight.el
 	 (when buffer-display-time
-	   (let* ((now (float-time))
-		  (then (float-time buffer-display-time)))
-	     (> (- now then) (* 60 60 ibuffer-old-time))))))))
+	   (time-less-p
+	    (* 60 60 ibuffer-old-time)
+	    (time-since buffer-display-time)))))))
 
 ;;;###autoload
 (defun ibuffer-mark-special-buffers ()
