@@ -1,6 +1,6 @@
 ;;; characters.el --- set syntax and category for multibyte characters
 
-;; Copyright (C) 1997, 2000-2020 Free Software Foundation, Inc.
+;; Copyright (C) 1997, 2000-2021 Free Software Foundation, Inc.
 ;; Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
 ;;   2005, 2006, 2007, 2008, 2009, 2010, 2011
 ;;   National Institute of Advanced Industrial Science and Technology (AIST)
@@ -674,6 +674,12 @@ with L, LRE, or LRO Unicode bidi character type.")
     (set-case-syntax c "." tbl)
     (setq c (1+ c)))
 
+  ;; Symbols for Legacy Computing
+  (setq c #x1FB00)
+  (while (<= c #x1FBFF)
+    (set-case-syntax c "." tbl)
+    (setq c (1+ c)))
+
   ;; Fullwidth Latin
   (setq c #xff21)
   (while (<= c #xff3a)
@@ -964,6 +970,7 @@ with L, LRE, or LRO Unicode bidi character type.")
 	   (#x0D41 . #x0D44)
 	   (#x0D4D . #x0D4D)
 	   (#x0D62 . #x0D63)
+           (#x0D81 . #x0D81)
 	   (#x0DCA . #x0DCA)
 	   (#x0DD2 . #x0DD6)
 	   (#x0E31 . #x0E31)
@@ -1020,7 +1027,7 @@ with L, LRE, or LRO Unicode bidi character type.")
 	   (#x1A65 . #x1A6C)
 	   (#x1A73 . #x1A7C)
 	   (#x1A7F . #x1A7F)
-	   (#x1AB0 . #x1ABE)
+	   (#x1AB0 . #x1AC0)
 	   (#x1B00 . #x1B03)
 	   (#x1B34 . #x1B34)
 	   (#x1B36 . #x1B3A)
@@ -1058,6 +1065,7 @@ with L, LRE, or LRO Unicode bidi character type.")
 	   (#xA806 . #xA806)
 	   (#xA80B . #xA80B)
 	   (#xA825 . #xA826)
+           (#xA82C . #xA82C)
 	   (#xA8C4 . #xA8C5)
 	   (#xA8E0 . #xA8F1)
 	   (#xA926 . #xA92D)
@@ -1094,6 +1102,7 @@ with L, LRE, or LRO Unicode bidi character type.")
 	   (#x10A01 . #x10A0F)
 	   (#x10A38 . #x10A3F)
 	   (#x10AE5 . #x10AE6)
+           (#x10EAB . #x10EAC)
 	   (#x11001 . #x11001)
 	   (#x11038 . #x11046)
 	   (#x1107F . #x11081)
@@ -1107,6 +1116,7 @@ with L, LRE, or LRO Unicode bidi character type.")
 	   (#x11180 . #x11181)
 	   (#x111B6 . #x111BE)
 	   (#x111CA . #x111CC)
+           (#x111CF . #x111CF)
 	   (#x1122F . #x11231)
 	   (#x11234 . #x11234)
 	   (#x11236 . #x11237)
@@ -1138,6 +1148,9 @@ with L, LRE, or LRO Unicode bidi character type.")
 	   (#x1171D . #x1171F)
 	   (#x11722 . #x11725)
 	   (#x11727 . #x1172B)
+           (#x1193B . #x1193C)
+           (#x1193E . #x1193E)
+           (#x11943 . #x11943)
 	   (#x11C30 . #x11C36)
 	   (#x11C38 . #x11C3D)
 	   (#x11C92 . #x11CA7)
@@ -1147,6 +1160,7 @@ with L, LRE, or LRO Unicode bidi character type.")
 	   (#x16AF0 . #x16AF4)
 	   (#x16B30 . #x16B36)
 	   (#x16F8F . #x16F92)
+           (#x16FE4 . #x16FE4)
 	   (#x1BC9D . #x1BC9E)
 	   (#x1BCA0 . #x1BCA3)
 	   (#x1D167 . #x1D169)
@@ -1210,7 +1224,7 @@ with L, LRE, or LRO Unicode bidi character type.")
 	   (#x2E80 . #x303E)
 	   (#x3040 . #x3247)
 	   (#x3250 . #x4DBF)
-	   (#x4E00 . #xA4CF)
+	   (#x4E00 . #x9FFF)
 	   (#xA490 . #xA4C6)
 	   (#xA960 . #xA97F)
 	   (#xAC00 . #xD7A3)
@@ -1219,9 +1233,11 @@ with L, LRE, or LRO Unicode bidi character type.")
 	   (#xFE30 . #xFE6F)
 	   (#xFF01 . #xFF60)
 	   (#xFFE0 . #xFFE6)
-	   (#x16FE0 . #x16FE3)
+	   (#x16FE0 . #x16FE4)
+           (#x16FF0 . #x16FF1)
 	   (#x17000 . #x187F7)
-	   (#x18800 . #x18AF2)
+	   (#x18800 . #x18AFF)
+           (#x18B00 . #x18CD5)
 	   (#x1B000 . #x1B152)
            (#x1B164 . #x1B167)
            (#x1B170 . #x1B2FB)
@@ -1229,6 +1245,7 @@ with L, LRE, or LRO Unicode bidi character type.")
 	   (#x1F0CF . #x1F0CF)
 	   (#x1F18E . #x1F18E)
 	   (#x1F191 . #x1F19A)
+           (#x1F1AD . #x1F1AD)
 	   (#x1F200 . #x1F320)
 	   (#x1F32D . #x1F335)
 	   (#x1F337 . #x1F37C)
@@ -1253,22 +1270,27 @@ with L, LRE, or LRO Unicode bidi character type.")
 	   (#x1F680 . #x1F6C5)
 	   (#x1F6CC . #x1F6CC)
 	   (#x1F6D0 . #x1F6D2)
-           (#x1F6D5 . #x1F6D5)
+           (#x1F6D5 . #x1F6D7)
 	   (#x1F6EB . #x1F6EC)
-	   (#x1F6F4 . #x1F6FA)
+	   (#x1F6F4 . #x1F6FC)
            (#x1F7E0 . #x1F7EB)
-	   (#x1F90D . #x1F971)
-	   (#x1F973 . #x1F976)
-	   (#x1F97A . #x1F9A2)
+	   (#x1F90C . #x1F93A)
+           (#x1F93C . #x1F945)
+           (#x1F947 . #x1F978)
+	   (#x1F97A . #x1F9CB)
            (#x1F9A5 . #x1F9AA)
            (#x1F9AE . #x1F9CA)
            (#x1F9CD . #x1F9FF)
            (#x1FA00 . #x1FA53)
            (#x1FA60 . #x1FA6D)
-           (#x1FA70 . #x1FA73)
+           (#x1FA70 . #x1FA74)
            (#x1FA78 . #x1FA7A)
-           (#x1FA80 . #x1FA82)
-           (#x1FA90 . #x1FA95)
+           (#x1FA80 . #x1FA86)
+           (#x1FA90 . #x1FAA8)
+           (#x1FAB0 . #x1FAB6)
+           (#x1FAC0 . #x1FAC2)
+           (#x1FAD0 . #x1FAD6)
+           (#x1FB00 . #x1FB92)
 	   (#x20000 . #x2FFFF)
 	   (#x30000 . #x3FFFF))))
   (dolist (elt l)
