@@ -1,6 +1,6 @@
 ;;; thai-util.el --- utilities for Thai -*- coding: utf-8; -*-
 
-;; Copyright (C) 2000-2017 Free Software Foundation, Inc.
+;; Copyright (C) 2000-2021 Free Software Foundation, Inc.
 ;; Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
 ;;   2005, 2006, 2007, 2008, 2009, 2010, 2011
 ;;   National Institute of Advanced Industrial Science and Technology (AIST)
@@ -225,10 +225,10 @@ positions (integers or markers) specifying the region."
   (thai-compose-region (point-min) (point-max)))
 
 ;;;###autoload
-(defun thai-composition-function (gstring)
+(defun thai-composition-function (gstring direction)
   (if (= (lgstring-char-len gstring) 1)
-      (compose-gstring-for-graphic gstring)
-    (or (font-shape-gstring gstring)
+      (compose-gstring-for-graphic gstring direction)
+    (or (font-shape-gstring gstring direction)
 	(let ((glyph-len (lgstring-glyph-len gstring))
 	      (last-char (lgstring-char gstring
 					(1- (lgstring-char-len gstring))))
@@ -256,11 +256,10 @@ positions (integers or markers) specifying the region."
 
 (define-minor-mode thai-word-mode
   "Minor mode to make word-oriented commands aware of Thai words.
-With a prefix argument ARG, enable the mode if ARG is positive,
-and disable it otherwise.  If called from Lisp, enable the mode
-if ARG is omitted or nil.  The commands affected are
-\\[forward-word], \\[backward-word], \\[kill-word], \\[backward-kill-word],
-\\[transpose-words], and \\[fill-paragraph]."
+
+The commands affected are \\[forward-word], \\[backward-word],
+\\[kill-word], \\[backward-kill-word], \\[transpose-words], and
+\\[fill-paragraph]."
   :global t :group 'mule
   (cond (thai-word-mode
 	 ;; This enables linebreak between Thai characters.
