@@ -1,6 +1,6 @@
 ;;; ps-bdf.el --- BDF font file handler for ps-print
 
-;; Copyright (C) 1998-1999, 2001-2017 Free Software Foundation, Inc.
+;; Copyright (C) 1998-1999, 2001-2021 Free Software Foundation, Inc.
 ;; Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
 ;;   2008, 2009, 2010, 2011
 ;;   National Institute of Advanced Industrial Science and Technology (AIST)
@@ -9,7 +9,7 @@
 ;;   National Institute of Advanced Industrial Science and Technology (AIST)
 ;;   Registration Number H13PRO009
 
-;; Author: Kenichi Handa <handa@m17n.org>
+;; Author: Kenichi Handa <handa@gnu.org>
 ;; (according to ack.texi)
 ;; Keywords: wp, BDF, font, PostScript
 ;; Package: ps-print
@@ -70,13 +70,12 @@ for BDFNAME."
 
 (defsubst bdf-file-mod-time (filename)
   "Return modification time of FILENAME.
-The value is a list of integers in the same format as `current-time'."
-  (nth 5 (file-attributes filename)))
+The value is a timestamp in the same format as `current-time'."
+  (file-attribute-modification-time (file-attributes filename)))
 
 (defun bdf-file-newer-than-time (filename mod-time)
   "Return non-nil if and only if FILENAME is newer than MOD-TIME.
-MOD-TIME is a modification time as a list of integers in the same
-format as `current-time'."
+MOD-TIME is a modification time in the same format as `current-time'."
   (let ((new-mod-time (bdf-file-mod-time filename)))
     (time-less-p mod-time new-mod-time)))
 
@@ -145,7 +144,7 @@ See the documentation of the function `bdf-read-font-info' for more detail."
   (if (or (< code (aref code-range 4))
 	  (> code (aref code-range 5)))
       (setq code (aref code-range 6)))
-  (+ (* (- (lsh code -8) (aref code-range 0))
+  (+ (* (- (ash code -8) (aref code-range 0))
 	(1+ (- (aref code-range 3) (aref code-range 2))))
      (- (logand code 255) (aref code-range 2))))
 
@@ -168,8 +167,7 @@ FONT-INFO is a list of the following format:
     (BDFFILE MOD-TIME FONT-BOUNDING-BOX
      RELATIVE-COMPOSE BASELINE-OFFSET CODE-RANGE MAXLEN OFFSET-VECTOR)
 
-MOD-TIME is last modification time as a list of integers in the
-same format as `current-time'.
+MOD-TIME is last modification time in the same format as `current-time'.
 
 SIZE is a size of the font on 72 dpi device.  This value is got
 from SIZE record of the font.
@@ -262,7 +260,7 @@ CODE, where N and CODE are in the following relation:
 	      (setq code (read (current-buffer)))
 	      (if (< code 0)
 		  (search-forward "ENDCHAR")
-		(setq code0 (lsh code -8)
+		(setq code0 (ash code -8)
 		      code1 (logand code 255)
 		      min-code (min min-code code)
 		      max-code (max max-code code)
