@@ -1,6 +1,6 @@
 /*
 Copyright (C) 1992 Lucid, Inc.
-Copyright (C) 2000-2019 Free Software Foundation, Inc.
+Copyright (C) 2000-2021 Free Software Foundation, Inc.
 
 This file is part of the Lucid Widget Library.
 
@@ -30,9 +30,8 @@ typedef struct _widget_instance
   Widget		widget;
   Widget		parent;
   Boolean		pop_up_p;
-#ifdef HAVE_XFT
+#if defined USE_CAIRO || defined HAVE_XFT
   struct widget_xft_data *xft_data;
-  int                   nr_xft_data;
 #endif
   struct _widget_info*		info;
   struct _widget_instance*	next;
