@@ -1,4 +1,4 @@
-/* Copyright (C) 1995, 2001-2019 Free Software Foundation, Inc.
+/* Copyright (C) 1995, 2001-2021 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -92,6 +92,8 @@ typedef unsigned short uint16_t;
 #define connect        sys_connect
 #define htons          sys_htons
 #define ntohs          sys_ntohs
+#define htonl          sys_htonl
+#define ntohl          sys_ntohl
 #define inet_addr      sys_inet_addr
 #define gethostname    sys_gethostname
 #define gethostbyname  sys_gethostbyname
@@ -112,6 +114,8 @@ int sys_bind (int s, const struct sockaddr *addr, int namelen);
 int sys_connect (int s, const struct sockaddr *addr, int namelen);
 u_short sys_htons (u_short hostshort);
 u_short sys_ntohs (u_short netshort);
+u_long sys_htonl (u_long hostlong);
+u_long sys_ntohl (u_long netlong);
 unsigned long sys_inet_addr (const char * cp);
 int sys_gethostname (char * name, int namelen);
 struct hostent * sys_gethostbyname (const char * name);
