@@ -1,4 +1,4 @@
-/* Copyright (C) 1985-1988, 1992-1994, 2001-2019 Free Software
+/* Copyright (C) 1985-1988, 1992-1994, 2001-2021 Free Software
  * Foundation, Inc.
 
 This file is part of GNU Emacs.
@@ -56,7 +56,7 @@ along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #define PERROR(file) report_error (file, new)
 
-#ifndef CANNOT_DUMP  /* all rest of file!  */
+#ifdef HAVE_UNEXEC  /* all rest of file!  */
 
 #ifdef HAVE_COFF_H
 #include <coff.h>
@@ -96,7 +96,6 @@ struct aouthdr
 #ifndef makedev			/* Try to detect types.h already loaded */
 #include <sys/types.h>
 #endif /* makedev */
-#include <stdio.h>
 #include <errno.h>
 
 #include <sys/file.h>
@@ -538,4 +537,4 @@ unexec (const char *new_name, const char *a_name)
     emacs_close (a_out);
 }
 
-#endif /* not CANNOT_DUMP */
+#endif /* HAVE_UNEXEC */
