@@ -1,6 +1,6 @@
 ;;; info-xref.el --- tests for info-xref.el
 
-;; Copyright (C) 2013-2019 Free Software Foundation, Inc.
+;; Copyright (C) 2013-2021 Free Software Foundation, Inc.
 
 ;; This file is part of GNU Emacs.
 
@@ -144,4 +144,21 @@ text.
                                (format "%s.info" (file-name-sans-extension
                                                   tempfile2)))))))
 
+(ert-deftest info-xref-test-emacs-manuals ()
+  "Test that all internal links in the Emacs manuals work."
+  :tags '(:expensive-test)
+  (require 'info)
+  (let ((default-directory (car (Info-default-dirs)))
+        (Info-directory-list '(".")))
+    (skip-unless (file-readable-p "emacs.info"))
+    (info-xref-check-all)
+    (with-current-buffer info-xref-output-buffer
+      (goto-char (point-max))
+      (should (search-backward "done" nil t))
+      (should (string-match-p
+               " [0-9]\\{3,\\} good, 0 bad"
+               (buffer-substring-no-properties (line-beginning-position)
+                                               (line-end-position)))))))
+
+
 ;;; info-xref.el ends here
